From c9cb43860ea43f754111bc11badded13115f0d1f Mon Sep 17 00:00:00 2001
From: Alyssa Rosenzweig <alyssa@rosenzweig.io>
Date: Sun, 28 Jan 2024 22:31:41 -0400
Subject: [PATCH] zink: honour nir->info.next_stage

to reduce compiling.

Signed-off-by: Alyssa Rosenzweig <alyssa@rosenzweig.io>
---
 src/gallium/drivers/zink/zink_compiler.c | 11 +++++++++--
 1 file changed, 9 insertions(+), 2 deletions(-)

diff --git a/src/gallium/drivers/zink/zink_compiler.c b/src/gallium/drivers/zink/zink_compiler.c
index c13106f516a..7281dd9223f 100644
--- a/src/gallium/drivers/zink/zink_compiler.c
+++ b/src/gallium/drivers/zink/zink_compiler.c
@@ -3170,8 +3170,15 @@ zink_shader_dump(const struct zink_shader *zs, void *words, size_t size, const c
 }
 
 static VkShaderStageFlagBits
-zink_get_next_stage(gl_shader_stage stage)
+zink_get_next_stage(gl_shader_stage stage, gl_shader_stage next_stage)
 {
+   /* If we have linked shaders in GL, we know which stage comes next so we can
+    * reduce redundant compiling of VS and TES. If we have separate shaders,
+    * next_stage will be 0 and we will need to allow all possible next stages.
+    */
+   if (next_stage)
+      return mesa_to_vk_shader_stage(next_stage);
+
    switch (stage) {
    case MESA_SHADER_VERTEX:
       return VK_SHADER_STAGE_TESSELLATION_CONTROL_BIT |
@@ -3211,7 +3218,7 @@ zink_shader_spirv_compile(struct zink_screen *screen, struct zink_shader *zs, st
 
    sci.sType = VK_STRUCTURE_TYPE_SHADER_CREATE_INFO_EXT;
    sci.stage = mesa_to_vk_shader_stage(zs->info.stage);
-   sci.nextStage = zink_get_next_stage(zs->info.stage);
+   sci.nextStage = zink_get_next_stage(zs->info.stage, zs->info.next_stage);
    sci.codeType = VK_SHADER_CODE_TYPE_SPIRV_EXT;
    sci.codeSize = spirv->num_words * sizeof(uint32_t);
    sci.pCode = spirv->words;
-- 
2.43.0

